package cz.jzitnik.game.handlers.place.handlers;

import cz.jzitnik.game.Block;
import cz.jzitnik.game.Game;
import cz.jzitnik.game.annotations.RegisterPlaceHandler;
import cz.jzitnik.game.handlers.place.CustomPlaceHandler;
import cz.jzitnik.game.items.ItemBlockSupplier;
import cz.jzitnik.game.sprites.OakDoor;
import cz.jzitnik.tui.ScreenRenderer;

@RegisterPlaceHandler("oak_door")
public class DoorPlaceHandler implements CustomPlaceHandler {
    public static void rightClick(Game game, int x, int y, ScreenRenderer screenRenderer) {
        var blocks = game.getWorld()[y][x];
        var door = blocks.stream().filter(block -> block.getBlockId().equals("oak_door")).toList().getFirst();


        switch (door.getSpriteState().get()) {
            case OakDoor.OakDoorState.TOP, OakDoor.OakDoorState.TOPCLOSED -> {
                var blocks2 = game.getWorld()[y+1][x];
                var door2 = blocks2.stream().filter(block -> block.getBlockId().equals("oak_door")).toList().getFirst();
                change(door2);
            }
            case OakDoor.OakDoorState.BOTTOM, OakDoor.OakDoorState.BOTTOMCLOSED -> {
                var blocks2 = game.getWorld()[y-1][x];
                var door2 = blocks2.stream().filter(block -> block.getBlockId().equals("oak_door")).toList().getFirst();
                change(door2);
            }
            default -> throw new IllegalStateException("Unexpected value: " + door.getSpriteState().get());
        }

        change(door);
        game.update(screenRenderer);
    }

    private static void change(Block door) {
        door.setSpriteState(switch (door.getSpriteState().get()) {
            case OakDoor.OakDoorState.TOP -> OakDoor.OakDoorState.TOPCLOSED;
            case OakDoor.OakDoorState.BOTTOM -> OakDoor.OakDoorState.BOTTOMCLOSED;
            case OakDoor.OakDoorState.TOPCLOSED -> OakDoor.OakDoorState.TOP;
            case OakDoor.OakDoorState.BOTTOMCLOSED -> OakDoor.OakDoorState.BOTTOM;
            default -> throw new IllegalStateException("Unexpected value: " + door.getSpriteState().get());
        });

        door.setGhost(switch (door.getSpriteState().get()) {
            case OakDoor.OakDoorState.TOP, OakDoor.OakDoorState.BOTTOM -> true;
            case OakDoor.OakDoorState.TOPCLOSED, OakDoor.OakDoorState.BOTTOMCLOSED -> false;
            default -> throw new IllegalStateException("Unexpected value: " + door.getSpriteState().get());
        });
    }

    @Override
    public boolean place(Game game, int x, int y) {
        var blocks = game.getWorld()[y][x];
        var blocksTop = game.getWorld()[y-1][x];

        if (!blocksTop.stream().allMatch(Block::isGhost)) {
            return false;
        }

        var inventory = game.getInventory();

        Block block = inventory.getItemInHand().get().getBlock().get();
        block.setSpriteState(OakDoor.OakDoorState.BOTTOMCLOSED);
        blocks.add(block);

        Block block2 = ItemBlockSupplier.Blocks.oakDoor();
        block2.setSpriteState(OakDoor.OakDoorState.TOPCLOSED);
        blocksTop.add(block2);

        inventory.decreaseItemInHand();

        return true;
    }

    @Override
    public void mine(Game game, int x, int y) {
        var blocks = game.getWorld()[y][x];
        Block block = blocks.stream().filter(b -> b.getBlockId().equals("oak_door")).toList().getFirst();

        if (block.getSpriteState().get() == OakDoor.OakDoorState.BOTTOM || block.getSpriteState().get() == OakDoor.OakDoorState.BOTTOMCLOSED) {
            var blocks2 = game.getWorld()[y-1][x];
            blocks2.removeAll(blocks2.stream().filter(i -> !i.getBlockId().equals("air")).toList());
        }

        if (block.getSpriteState().get() == OakDoor.OakDoorState.TOP || block.getSpriteState().get() == OakDoor.OakDoorState.TOPCLOSED) {
            var blocks2 = game.getWorld()[y+1][x];
            blocks2.removeAll(blocks2.stream().filter(i -> !i.getBlockId().equals("air")).toList());
        }

        blocks.removeAll(blocks.stream().filter(i -> !i.getBlockId().equals("air")).toList());
    }
}
